<?php

namespace Drupal\commerce_sameday\Plugin\Action;

use Sameday\Objects\PostAwb\Request\AwbRecipientEntityObject;
use Sameday\Objects\Types\AwbPaymentType;
use Sameday\Objects\ParcelDimensionsObject;
use Sameday\Objects\Types\PackageType;
use Drupal\Core\Action\ActionBase;
use Drupal\Core\Session\AccountInterface;
use Drupal\commerce_shipping\Entity\ShipmentInterface;
use Drupal\Core\Messenger\MessengerTrait;

/**
 * Generates a Sameday AWB for a shipment.
 *
 * @Action(
 *   id = "commerce_sameday_generate_awb",
 *   label = @Translation("Generate Sameday AWB"),
 *   type = "commerce_shipment"
 * )
 */
class GenerateSamedayAwb extends ActionBase {
  use MessengerTrait;

  /**
   * {@inheritdoc}
   */
  public function execute($entity = NULL) {
    if (!$entity instanceof ShipmentInterface) {
      return;
    }
    $order = $entity->getOrder();
    $shipping_method = $entity->getShippingMethod();
    if (!$shipping_method) {
      $this->messenger()->addError(t('No shipping method found for this shipment.'));
      return;
    }
    $plugin = $shipping_method->getPlugin();
    // Prepare shipment details (customize as needed for your site).
    $shipment_details = [
      'pickup_point_id' => 0,
      'package_type' => new PackageType(PackageType::PARCEL),
      'parcels' => [new ParcelDimensionsObject(1, 10, 10, 10)],
      'service_id' => $shipping_method->get('plugin_configuration')['allowed_services'][0] ?? NULL,
      'awb_payment' => new AwbPaymentType(2),
    // You should build a real recipient object from the shipment address.
      'recipient' => NULL,
    ];
    // Example: Build recipient from shipment address (customize as needed).
    $address = $entity->getShippingProfile()->get('address')->first();
    if ($address) {
      $shipment_details['recipient'] = new AwbRecipientEntityObject(
        $address->getLocality(),
        $address->getAdministrativeArea(),
        $address->getAddressLine1(),
        $address->getGivenName() . ' ' . $address->getFamilyName(),
      // Phone.
        '',
      // Email.
        '',
        NULL,
        $address->getPostalCode()
      );
    }
    $awb = $plugin->generateAwb($order, $shipment_details);
    if ($awb) {
      $entity->setData('sameday_awb', $awb);
      $entity->save();
      $this->messenger()->addStatus(t('Sameday AWB generated: @awb', ['@awb' => $awb]));
    }
    else {
      $this->messenger()->addError(t('Failed to generate Sameday AWB.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function access($object, ?AccountInterface $account = NULL, $return_as_object = FALSE) {
    return $object instanceof ShipmentInterface && $object->access('update', $account, $return_as_object);
  }

}
