<?php

namespace Drupal\commerce_sameday\ShippingMethod;

use Sameday\Requests\SamedayGetAwbStatusHistoryRequest;
use Sameday\Requests\SamedayPostAwbRequest;
use Drupal\Core\Form\FormStateInterface;
use Drupal\commerce_shipping\Plugin\Commerce\ShippingMethod\ShippingMethodBase;
use Drupal\commerce_shipping\Entity\ShipmentInterface;
use Sameday\SamedayClient;
use Sameday\Request\SamedayGetServicesRequest;
use Sameday\Exceptions\SamedayException;
use Drupal\commerce_shipping\Plugin\Commerce\ShippingMethod\ShippingRate;
use Drupal\commerce_price\Price;
use Sameday\Sameday;
use Sameday\Requests\SamedayPostAwbEstimationRequest;
use Sameday\Requests\SamedayGetLockersRequest;
use Sameday\Objects\Types\AwbPaymentType;
use Sameday\Objects\Types\PackageType;
use Sameday\Objects\ParcelDimensionsObject;
use Sameday\Objects\PostAwb\Request\AwbRecipientEntityObject;

/**
 * Provides the Sameday shipping method plugin.
 *
 * @CommerceShippingMethod(
 *   id = "sameday",
 *   label = @Translation("Sameday Courier"),
 *   services = {},
 * )
 */
class SamedayShippingMethod extends ShippingMethodBase {

  /**
   * {@inheritdoc}
   */
  public function calculateRates(ShipmentInterface $shipment) {
    $rates = [];
    try {
      $config = $this->getConfiguration();
      $username = $config['username'] ?? '';
      $password = $config['password'] ?? '';
      $clientId = $config['client_id'] ?? '';
      $clientSecret = $config['client_secret'] ?? '';
      $sandbox = !empty($config['sandbox']);
      $allowed_services = $config['allowed_services'] ?? [];

      $client = new SamedayClient(
        $username,
        $password,
        $clientId,
        $clientSecret,
        $sandbox
      );
      $sameday = new Sameday($client);
      $servicesRequest = new SamedayGetServicesRequest();
      $servicesResponse = $sameday->getServices($servicesRequest);
      $services = $servicesResponse->getServices();

      foreach ($services as $service) {
        if (!empty($allowed_services) && !in_array($service->getId(), $allowed_services)) {
          continue;
        }
        // --- Real Dynamic Rate Calculation ---
        $shipment_address = $shipment->getShippingProfile()->get('address')->first();
        $weight = $shipment->getWeight() ? $shipment->getWeight()->getNumber() : 1;
        $city = $shipment_address ? $shipment_address->getLocality() : '';
        $county = $shipment_address ? $shipment_address->getAdministrativeArea() : '';
        $country = $shipment_address ? $shipment_address->getCountryCode() : '';
        $recipient = new AwbRecipientEntityObject(
          $shipment_address ? $shipment_address->getGivenName() : '',
          $shipment_address ? $shipment_address->getFamilyName() : '',
          $city,
          $county,
          $country,
          $shipment_address ? $shipment_address->getAddressLine1() : '',
          $shipment_address ? $shipment_address->getPostalCode() : '',
        // Phone.
          '',
        // Email.
          '',
        // Company.
          ''
        );
        $parcels = [
        // Use real dims if available.
          new ParcelDimensionsObject($weight, 10, 10, 10),
        ];
        $packageType = new PackageType(PackageType::PARCEL);
        // 2 = recipient
        $awbPayment = new AwbPaymentType(2);
        // Set your pickup point ID if available/configured.
        $pickupPointId = 0;
        $estimationRequest = new SamedayPostAwbEstimationRequest(
          $pickupPointId,
        // Contact person.
          NULL,
          $packageType,
          $parcels,
          $service->getId(),
          $awbPayment,
          $recipient,
        // Insured value.
          0,
        // Cash on delivery.
          0
        );
        try {
          $estimationResponse = $sameday->postAwbEstimation($estimationRequest);
          $price_amount = $estimationResponse->getCost();
          $price_currency = $estimationResponse->getCurrency();
        }
        catch (\Exception $e) {
          $price_amount = 0;
          $price_currency = 'BGN';
        }
        $price = new Price((string) $price_amount, $price_currency);
        $rate = new ShippingRate(
          $service->getName(),
          $this->t($service->getName()),
          $price,
          $this->t('Delivery with Sameday: @service', ['@service' => $service->getName()]),
          TRUE
        );
        $rates[] = $rate;
        // --- End Real Dynamic Rate Calculation ---
      }
    }
    catch (SamedayException $e) {
      \Drupal::logger('commerce_sameday')->error('Sameday API error: @message', ['@message' => $e->getMessage()]);
    }
    catch (\Exception $e) {
      \Drupal::logger('commerce_sameday')->error('General error: @message', ['@message' => $e->getMessage()]);
    }
    return $rates;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'username' => '',
      'password' => '',
      'client_id' => '',
      'client_secret' => '',
      'sandbox' => TRUE,
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['username'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Sameday Username'),
      '#default_value' => $this->configuration['username'],
      '#required' => TRUE,
    ];
    $form['password'] = [
      '#type' => 'password',
      '#title' => $this->t('Sameday Password'),
      '#default_value' => $this->configuration['password'],
      '#required' => TRUE,
    ];
    $form['client_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Client ID'),
      '#default_value' => $this->configuration['client_id'],
      '#required' => TRUE,
    ];
    $form['client_secret'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Client Secret'),
      '#default_value' => $this->configuration['client_secret'],
      '#required' => TRUE,
    ];
    $form['sandbox'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use Sandbox'),
      '#default_value' => $this->configuration['sandbox'],
    ];
    // --- Service Filtering Feature ---
    // Fetch available Sameday services for selection (if credentials are set).
    $service_options = [];
    try {
      if (!empty($this->configuration['username']) && !empty($this->configuration['password']) && !empty($this->configuration['client_id']) && !empty($this->configuration['client_secret'])) {
        $sameday = new SamedayClient(
          $this->configuration['username'],
          $this->configuration['password'],
          $this->configuration['client_id'],
          $this->configuration['client_secret'],
          !empty($this->configuration['sandbox'])
        );
        $servicesRequest = new SamedayGetServicesRequest();
        $servicesResponse = $sameday->getServices($servicesRequest);
        foreach ($servicesResponse->getServices() as $service) {
          $service_options[$service->getId()] = $service->getName();
        }
      }
    }
    catch (\Exception $e) {
      $service_options = [];
    }
    $form['allowed_services'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Allowed Sameday Services'),
      '#options' => $service_options,
      '#default_value' => $this->configuration['allowed_services'] ?? [],
      '#description' => $this->t('Select which Sameday services to offer as shipping options.'),
    ];
    // --- End Service Filtering Feature ---
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $values = $form_state->getValues();
    $this->configuration['username'] = $values['username'];
    $this->configuration['password'] = $values['password'];
    $this->configuration['client_id'] = $values['client_id'];
    $this->configuration['client_secret'] = $values['client_secret'];
    $this->configuration['sandbox'] = $values['sandbox'];
    $this->configuration['allowed_services'] = array_filter($values['allowed_services'] ?? []);
  }

  /**
   * Returns a list of available Sameday lockers for the given shipment address.
   *
   * @param string $city
   *   The city for delivery.
   * @param string $county
   *   The county for delivery.
   * @param string $country
   *   The country code.
   *
   * @return array
   *   Locker ID => Locker name.
   */
  protected function getAvailableLockers($city, $county, $country) {
    $lockers = [];
    try {
      $config = $this->getConfiguration();
      $client = new SamedayClient(
        $config['username'],
        $config['password'],
        $config['client_id'],
        $config['client_secret'],
        !empty($config['sandbox'])
      );
      $sameday = new Sameday($client);
      $lockersRequest = new SamedayGetLockersRequest();
      $lockersResponse = $sameday->getLockers($lockersRequest);
      foreach ($lockersResponse->getLockers() as $locker) {
        // Optionally filter by city/county/country.
        if (
          ($city && strcasecmp($locker->getCity(), $city) !== 0) ||
          ($county && strcasecmp($locker->getCounty(), $county) !== 0) ||
          ($country && strcasecmp($locker->getCountry(), $country) !== 0)
        ) {
          continue;
        }
        $lockers[$locker->getId()] = $locker->getName() . ' - ' . $locker->getAddress();
      }
    }
    catch (\Exception $e) {
      // Log or handle error.
    }
    return $lockers;
  }

  /**
   * Generates a Sameday AWB for the given order and shipment details.
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $order
   *   The order entity.
   * @param array $shipment_details
   *   The shipment details (address, weight, service, locker, etc.).
   *
   * @return string|null
   *   The AWB number, or NULL on failure.
   */
  public function generateAwb($order, array $shipment_details) {
    try {
      $config = $this->getConfiguration();
      $client = new SamedayClient(
        $config['username'],
        $config['password'],
        $config['client_id'],
        $config['client_secret'],
        !empty($config['sandbox'])
      );
      $sameday = new Sameday($client);
      // Extract shipment details (ensure all required fields are present)
      $pickupPointId = $shipment_details['pickup_point_id'] ?? 0;
      $contactPersonId = $shipment_details['contact_person_id'] ?? NULL;
      $packageType = $shipment_details['package_type'] ?? new PackageType(PackageType::PARCEL);
      $parcels = $shipment_details['parcels'] ?? [new ParcelDimensionsObject(1, 10, 10, 10)];
      $serviceId = $shipment_details['service_id'] ?? NULL;
      // 2 = recipient
      $awbPayment = $shipment_details['awb_payment'] ?? new AwbPaymentType(2);
      $recipient = $shipment_details['recipient'] ?? NULL;
      $insuredValue = $shipment_details['insured_value'] ?? 0;
      $codAmount = $shipment_details['cod_amount'] ?? 0;
      $lockerFirstMile = $shipment_details['locker_first_mile'] ?? NULL;
      $lockerLastMile = $shipment_details['locker_last_mile'] ?? NULL;
      if (!$serviceId || !$recipient) {
        throw new \Exception('Missing required AWB shipment details.');
      }
      $awbRequest = new SamedayPostAwbRequest(
        $pickupPointId,
        $contactPersonId,
        $packageType,
        $parcels,
        $serviceId,
        $awbPayment,
        $recipient,
        $insuredValue,
        $codAmount,
      // CodCollectorType.
        NULL,
      // ThirdPartyPickupEntityObject.
        NULL,
      // serviceTaxIds.
        [],
      // DeliveryIntervalServiceType.
        NULL,
        $shipment_details['reference'] ?? NULL,
        $shipment_details['observation'] ?? NULL,
        $shipment_details['price_observation'] ?? NULL,
        $shipment_details['client_observation'] ?? NULL,
        $lockerFirstMile,
        $lockerLastMile
      );
      $awbResponse = $sameday->postAwb($awbRequest);
      return $awbResponse->getAwbNumber();
    }
    catch (\Exception $e) {
      \Drupal::logger('commerce_sameday')->error('AWB generation failed: @msg', ['@msg' => $e->getMessage()]);
      return NULL;
    }
  }

  /**
   * Gets the tracking status for a Sameday AWB.
   *
   * @param string $awb_number
   *   The AWB number to track.
   *
   * @return array|null
   *   An array with status and history, or NULL on failure.
   */
  public function getAwbTracking($awb_number) {
    try {
      $config = $this->getConfiguration();
      $client = new SamedayClient(
        $config['username'],
        $config['password'],
        $config['client_id'],
        $config['client_secret'],
        !empty($config['sandbox'])
      );
      $sameday = new Sameday($client);
      $trackingRequest = new SamedayGetAwbStatusHistoryRequest($awb_number);
      $trackingResponse = $sameday->getAwbStatusHistory($trackingRequest);
      $summary = $trackingResponse->getSummary();
      $history = $trackingResponse->getHistory();
      $status = $summary ? $summary->getServicePayment() : '';
      $history_array = [];
      foreach ($history as $event) {
        $history_array[] = [
          'date' => $event->getDate()->format('Y-m-d H:i:s'),
          'event' => $event->getLabel(),
        ];
      }
      return [
        'status' => $status,
        'history' => $history_array,
      ];
    }
    catch (\Exception $e) {
      \Drupal::logger('commerce_sameday')->error('AWB tracking failed: @msg', ['@msg' => $e->getMessage()]);
      return NULL;
    }
  }

  // --- Integration points for UI (pseudo-code, to be implemented in forms/controllers) ---
  // 1. Checkout UI: Locker selection
  //    - Add a form element in the checkout pane for locker selection, using getAvailableLockers().
  //   - Save the selected locker to the shipment/order entity.
  //
  // 2. Admin UI: AWB generation
  //    - Add an order operation or shipment action to call generateAwb().
  //   - Save the returned AWB number to the order/shipment entity.
  //
  // 3. Admin/Customer UI: Tracking
  //    - Display the current status and history from getAwbTracking() on the order/shipment view page.
  //
  // See README for code snippets and usage examples.
}
